#define QT_FEATURE_cross_compile -1
#define QT_FEATURE_framework -1
#define QT_FEATURE_shared 1
#define QT_FEATURE_rpath 1
#define QT_FEATURE_appstore_compliant -1
#define QT_FEATURE_debug_and_release -1
#define QT_FEATURE_simulator_and_device -1
#define QT_FEATURE_build_all -1
#define QT_FEATURE_c__11 1
#define QT_FEATURE_c__14 1
#define QT_FEATURE_c__1z 1
#define QT_FEATURE_concurrent 1
#define QT_FEATURE_pkg_config 1
#define QT_FEATURE_force_asserts -1
#define QT_LARGEFILE_SUPPORT 64
#define QT_VISIBILITY_AVAILABLE true
#define QT_REDUCE_RELOCATIONS true
#define QT_FEATURE_separate_debug_info -1
#define QT_COMPILER_SUPPORTS_SSE2 1
#define QT_COMPILER_SUPPORTS_SSE3 1
#define QT_COMPILER_SUPPORTS_SSSE3 1
#define QT_COMPILER_SUPPORTS_SSE4_1 1
#define QT_COMPILER_SUPPORTS_SSE4_2 1
#define QT_FEATURE_static -1
#define QT_VERSION_STR "5.9.7"
#define QT_VERSION_MAJOR 5
#define QT_VERSION_MINOR 9
#define QT_VERSION_PATCH 7
