/* This file was generated by syncqt. */
#ifndef QT_QTSERIALPORT_VERSION_H
#define QT_QTSERIALPORT_VERSION_H

#define QTSERIALPORT_VERSION_STR "5.9.7"

#define QTSERIALPORT_VERSION 0x050907

#endif // QT_QTSERIALPORT_VERSION_H
