/* This file was generated by syncqt. */
#ifndef QT_QTINPUTSUPPORT_VERSION_H
#define QT_QTINPUTSUPPORT_VERSION_H

#define QTINPUTSUPPORT_VERSION_STR "5.9.7"

#define QTINPUTSUPPORT_VERSION 0x050907

#endif // QT_QTINPUTSUPPORT_VERSION_H
