/* This file was generated by syncqt. */
#ifndef QT_QTX11EXTRAS_VERSION_H
#define QT_QTX11EXTRAS_VERSION_H

#define QTX11EXTRAS_VERSION_STR "5.9.7"

#define QTX11EXTRAS_VERSION 0x050907

#endif // QT_QTX11EXTRAS_VERSION_H
