/* This file was generated by syncqt. */
#ifndef QT_QTQUICKCONTROLS2_VERSION_H
#define QT_QTQUICKCONTROLS2_VERSION_H

#define QTQUICKCONTROLS2_VERSION_STR "5.9.7"

#define QTQUICKCONTROLS2_VERSION 0x050907

#endif // QT_QTQUICKCONTROLS2_VERSION_H
